package livework;
/* (c) Alex McLean 2006
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

/**
 * <p>A simple but useful envelope class.  Tweens between a given a set of discreet 
 * values that represent an envelope, so that a value can be returned for a 
 * floating point percentage.  Tweening is linear.</p>
 * <p>This allows a set of envelopes to be applied to a visual element, for example to 
 * control brightness - fading in and out.
 * </p>
 * <p>The given array of values should be considered as continuous, equally spaced
 * points describing the envelope.
 * </p>
 */
public class Envelope {
	/** number of values */
	private int count;
	/** array of values */
	private float[] values;

	/**
	 * Constructor for this class
	 * @param values array of float values.
	 */
	Envelope (float[] values) {
		this.values = values;
	    count = values.length;
	}

	/**
	 * Calculates a value of the given percentage
	 * @param percentage
	 * @return the calculated value 
	 */
	float value (float percentage) {
		float result;
		// normalise percentage;
		if (percentage < 0) {
			percentage = 0;
		}
		else if (percentage > 1) {
			percentage = 1;
		}
			    
	    if (percentage == 0) {
	    	result = values[0];
	    }
	    else if (percentage == 1) {
	    	result = values[count - 1];
	    }
	    else {
		    float diff, point;	    
		    point = (count - 1) * percentage;
	    	int a, b;
	    	a = (int) point;
	    	b = (int) point + 1;
	    	if (a < 0)  {
	    		a = 0;
	    	}
	    	if (b > (count - 1)) {
	    		b = count - 1;
	    	}
	    	diff = values[b] - values[a];
	    	result = values[a] + (diff * (point - a));
	    }
	    return(result);
	}
}
