package livework;
/* (c) Alex McLean 2006
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

import java.awt.*;
import java.awt.geom.*;

/**
 * A class representing a circle of a given colour, brightness envelope,  lifespan (ttl), location and size.
 * @see Nourathar
 * @author  Alex McLean - ma503am@gold.ac.uk
 */
public class NouratharColor extends Ellipse2D.Double {
	/** age of this object in triggers */
	private int age;
	/** Color of this object */
	private Color color;
	/* brightness envelope */
	private Envelope envelope;
	/* time to live, or lifespan of this object in triggers */
	private int ttl;

	/**
	 * Constructor for this object
	 */
	NouratharColor(Color color, Envelope envelope, int ttl, double x, double y, double size) {
		super(x, y, size, size);
		this.color = color;
		this.envelope = envelope;
		this.ttl = ttl;
		age = 0;
	}
	
	
	/**
	 * 'Dampens' the colour of this object by finding the current value of the 
	 * brightness envelope, and applying it to the alpha component of the colour.
	 * @return the new colour
	 */
	private Color dampenedColor() {
		// Five method calls - perhaps there is a more efficent way of doing this?
		Color newColor = new Color(color.getRed(), color.getBlue(), color.getGreen(),
				                   (int) (color.getAlpha() * getStrength())
				                  );
		return(newColor);
	}
	
	/**
	 * Draws the colour, at its predefined location and size, and using the colour
	 * calculated using the predefined brightness envelope.
	 * @param g2d
	 */
	void draw(Graphics2D g2d) {
		Color origColor = g2d.getColor();
		g2d.setColor(dampenedColor());
		g2d.fill(this);
		g2d.setColor(origColor);
		
		// make a bit older
		age++;
	}
	
	/** 
	 * Calculates the current strength (or brightness) of the colour according
	 * to the brightness envelope
	 * @return The calculated value
	 */
	
	double getStrength() {
		return(envelope.value((float) age / ttl));
	}
	
	/**
	 * Calculates whether this object is as old or order than it's given ttl 
	 * (lifespan)
	 * @return The calculated value
	 */
	boolean isExpired() {
		return(age >= ttl);
	}
}
